
'''Module of one function: my_mkdir,which creates directories as needed'''

import os
import re
import sys
import errno

import python2x3
import constants_mod


def my_mkdir(pathname):
    # pylint: disable=W0142
    # W0142: We want * magic

    '''
    Create one or more directories specified by directory_list.
    If pathname has [ 'a', 'b', 'c' ], then we create a/b/c.
    If pathname has 'a/b/c' then we create a/b/c
    '''

    bytes_path_sep = python2x3.string_to_binary(os.path.sep)

    if isinstance(pathname, str):
        complete_relative_path = python2x3.string_to_binary(pathname)
        directory_list = python2x3.string_to_binary(pathname).split(bytes_path_sep)
    elif isinstance(pathname, python2x3.bytes_type):
        # Sometimes this'll be str - handled above
        complete_relative_path = pathname
        directory_list = pathname.split(bytes_path_sep)
    elif isinstance(pathname, list):
        complete_relative_path = os.path.join(*pathname)
        directory_list = pathname
    else:
        raise ValueError('pathname is not str, bytes or list')

    if os.path.isdir(complete_relative_path):
        return

    for final_position in range(1, len(directory_list) + 1):
        partial_relative_slice = directory_list[:final_position]
        partial_relative_path = os.path.join(*partial_relative_slice)
        try:
            os.mkdir(partial_relative_path, 7 * 64 + 5 * 8 + 5)
        except OSError:
            dummy, mkdir_extra, dummy = sys.exc_info()
            if mkdir_extra.errno == errno.EEXIST:
                continue
            else:
                raise


def prepend_dirs(path):
    # pylint: disable=W0142
    # W0142: I like * magic
    '''Take a relative path and add dir- prefixes to all the directories'''
    path_parts, filename = get_path_parts(path)
    directories = path_parts[:]
    # filename is a directory too in this case
    directories.append(filename)
    nonempty_directories = [ directory for directory in directories if directory and directory != constants_mod.Constants.b_dot ]
    dired_directories = [ 'dir-%s' % directory for directory in nonempty_directories ]
    if not dired_directories:
        dired_directories = [ '.' ]
    result = os.path.join(*dired_directories)
    return result


def strip_dirs(pathname):
    '''Strip off all the dir- prefixes'''
    path1 = re.sub(constants_mod.Constants.b_slash_dir_dash, constants_mod.Constants.b_slash, python2x3.string_to_binary(pathname))
    path2 = re.sub(constants_mod.Constants.b_hat_dot_slash, python2x3.empty_bytes, path1)
    path3 = re.sub(constants_mod.Constants.b_hat_dir_dash, python2x3.empty_bytes, path2)
    return path3


def get_path_parts(relative_path):
    '''Chop a (directory-containing) pathname into the directory leading up to it (dirname), and its final part (basename)'''
    if isinstance(relative_path, list):
        path_components = relative_path
    else:
        path_components = python2x3.string_to_binary(relative_path).split(python2x3.string_to_binary(os.path.sep))
    nonempty_path_components = [ component for component in path_components if component != python2x3.empty_bytes ]
    relative_directories = nonempty_path_components[:-1]
    if nonempty_path_components:
        filename = nonempty_path_components[-1]
    else:
        filename = python2x3.string_to_binary(os.path.sep)

    return (relative_directories, filename)

